/*
 * Driver for the Atmel USBA high speed USB device controller
 *
 * Copyright (C) 2005-2007 Atmel Corporation
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */
#ifndef __LINUX_USB_GADGET_USBA_UDC_H__
#define __LINUX_USB_GADGET_USBA_UDC_H__

/* USB register offsets */
#define USBA_CTRL				0x0000
#define USBA_FNUM				0x0004
#define USBA_INT_ENB				0x0010
#define USBA_INT_STA				0x0014
#define USBA_INT_CLR				0x0018
#define USBA_EPT_RST				0x001c
#define USBA_TST_SOF_CNT			0x00d0
#define USBA_TST_CNT_A				0x00d4
#define USBA_TST_CNT_B				0x00d8
#define USBA_TST_MODE_REG			0x00dc
#define USBA_TST				0x00e0

/* USB endpoint register offsets */
#define USBA_EPT_CFG				0x0000
#define USBA_EPT_CTL_ENB			0x0004
#define USBA_EPT_CTL_DIS			0x0008
#define USBA_EPT_CTL				0x000c
#define USBA_EPT_SET_STA			0x0014
#define USBA_EPT_CLR_STA			0x0018
#define USBA_EPT_STA				0x001c

/* USB DMA register offsets */
#define USBA_DMA_NXT_DSC			0x0000
#define USBA_DMA_ADDRESS			0x0004
#define USBA_DMA_CONTROL			0x0008
#define USBA_DMA_STATUS				0x000c

/* Bitfields in CTRL */
#define USBA_DEV_ADDR_OFFSET			0
#define USBA_DEV_ADDR_SIZE			7
#define USBA_FADDR_EN_OFFSET			7
#define USBA_FADDR_EN_SIZE			1
#define USBA_EN_USBA_OFFSET			8
#define USBA_EN_USBA_SIZE			1
#define USBA_DETACH_OFFSET			9
#define USBA_DETACH_SIZE			1
#define USBA_REMOTE_WAKE_UP_OFFSET		10
#define USBA_REMOTE_WAKE_UP_SIZE		1

/* Bitfields in FNUM */
#define USBA_MICRO_FRAME_NUM_OFFSET		0
#define USBA_MICRO_FRAME_NUM_SIZE		3
#define USBA_FRAME_NUMBER_OFFSET		3
#define USBA_FRAME_NUMBER_SIZE			11
#define USBA_FRAME_NUM_ERROR_OFFSET		31
#define USBA_FRAME_NUM_ERROR_SIZE		1

/* Bitfields in INT_ENB/INT_STA/INT_CLR */
#define USBA_HIGH_SPEED_OFFSET			0
#define USBA_HIGH_SPEED_SIZE			1
#define USBA_DET_SUSPEND_OFFSET			1
#define USBA_DET_SUSPEND_SIZE			1
#define USBA_MICRO_SOF_OFFSET			2
#define USBA_MICRO_SOF_SIZE			1
#define USBA_SOF_OFFSET				3
#define USBA_SOF_SIZE				1
#define USBA_END_OF_RESET_OFFSET		4
#define USBA_END_OF_RESET_SIZE			1
#define USBA_WAKE_UP_OFFSET			5
#define USBA_WAKE_UP_SIZE			1
#define USBA_END_OF_RESUME_OFFSET		6
#define USBA_END_OF_RESUME_SIZE			1
#define USBA_UPSTREAM_RESUME_OFFSET		7
#define USBA_UPSTREAM_RESUME_SIZE		1
#define USBA_EPT_INT_OFFSET			8
#define USBA_EPT_INT_SIZE			16
#define USBA_DMA_INT_OFFSET			24
#define USBA_DMA_INT_SIZE			8

/* Bitfields in EPT_RST */
#define USBA_RST_OFFSET				0
#define USBA_RST_SIZE				16

/* Bitfields in TST_SOF_CNT */
#define USBA_SOF_CNT_MAX_OFFSET			0
#define USBA_SOF_CNT_MAX_SIZE			7
#define USBA_SOF_CNT_LOAD_OFFSET		7
#define USBA_SOF_CNT_LOAD_SIZE			1

/* Bitfields in TST_CNT_A */
#define USBA_CNT_A_MAX_OFFSET			0
#define USBA_CNT_A_MAX_SIZE			7
#define USBA_CNT_A_LOAD_OFFSET			7
#define USBA_CNT_A_LOAD_SIZE			1

/* Bitfields in TST_CNT_B */
#define USBA_CNT_B_MAX_OFFSET			0
#define USBA_CNT_B_MAX_SIZE			7
#define USBA_CNT_B_LOAD_OFFSET			7
#define USBA_CNT_B_LOAD_SIZE			1

/* Bitfields in TST_MODE_REG */
#define USBA_TST_MODE_OFFSET			0
#define USBA_TST_MODE_SIZE			6

/* Bitfields in USBA_TST */
#define USBA_SPEED_CFG_OFFSET			0
#define USBA_SPEED_CFG_SIZE			2
#define USBA_TST_J_MODE_OFFSET			2
#define USBA_TST_J_MODE_SIZE			1
#define USBA_TST_K_MODE_OFFSET			3
#define USBA_TST_K_MODE_SIZE			1
#define USBA_TST_PKT_MODE_OFFSET		4
#define USBA_TST_PKT_MODE_SIZE			1
#define USBA_OPMODE2_OFFSET			5
#define USBA_OPMODE2_SIZE			1

/* Bitfields in EPT_CFG */
#define USBA_EPT_SIZE_OFFSET			0
#define USBA_EPT_SIZE_SIZE			3
#define USBA_EPT_DIR_OFFSET			3
#define USBA_EPT_DIR_SIZE			1
#define USBA_EPT_TYPE_OFFSET			4
#define USBA_EPT_TYPE_SIZE			2
#define USBA_BK_NUMBER_OFFSET			6
#define USBA_BK_NUMBER_SIZE			2
#define USBA_NB_TRANS_OFFSET			8
#define USBA_NB_TRANS_SIZE			2
#define USBA_EPT_MAPPED_OFFSET			31
#define USBA_EPT_MAPPED_SIZE			1

/* Bitfields in EPT_CTL/EPT_CTL_ENB/EPT_CTL_DIS */
#define USBA_EPT_ENABLE_OFFSET			0
#define USBA_EPT_ENABLE_SIZE			1
#define USBA_AUTO_VALID_OFFSET			1
#define USBA_AUTO_VALID_SIZE			1
#define USBA_INTDIS_DMA_OFFSET			3
#define USBA_INTDIS_DMA_SIZE			1
#define USBA_NYET_DIS_OFFSET			4
#define USBA_NYET_DIS_SIZE			1
#define USBA_DATAX_RX_OFFSET			6
#define USBA_DATAX_RX_SIZE			1
#define USBA_MDATA_RX_OFFSET			7
#define USBA_MDATA_RX_SIZE			1
/* Bits 8-15 and 31 enable interrupts for respective bits in EPT_STA */
#define USBA_BUSY_BANK_IE_OFFSET		18
#define USBA_BUSY_BANK_IE_SIZE			1

/* Bitfields in EPT_SET_STA/EPT_CLR_STA/EPT_STA */
#define USBA_FORCE_STALL_OFFSET			5
#define USBA_FORCE_STALL_SIZE			1
#define USBA_TOGGLE_SEQ_OFFSET			6
#define USBA_TOGGLE_SEQ_SIZE			2
#define USBA_ERR_OVFLW_OFFSET			8
#define USBA_ERR_OVFLW_SIZE			1
#define USBA_RX_BK_RDY_OFFSET			9
#define USBA_RX_BK_RDY_SIZE			1
#define USBA_KILL_BANK_OFFSET			9
#define USBA_KILL_BANK_SIZE			1
#define USBA_TX_COMPLETE_OFFSET			10
#define USBA_TX_COMPLETE_SIZE			1
#define USBA_TX_PK_RDY_OFFSET			11
#define USBA_TX_PK_RDY_SIZE			1
#define USBA_ISO_ERR_TRANS_OFFSET		11
#define USBA_ISO_ERR_TRANS_SIZE			1
#define USBA_RX_SETUP_OFFSET			12
#define USBA_RX_SETUP_SIZE			1
#define USBA_ISO_ERR_FLOW_OFFSET		12
#define USBA_ISO_ERR_FLOW_SIZE			1
#define USBA_STALL_SENT_OFFSET			13
#define USBA_STALL_SENT_SIZE			1
#define USBA_ISO_ERR_CRC_OFFSET			13
#define USBA_ISO_ERR_CRC_SIZE			1
#define USBA_ISO_ERR_NBTRANS_OFFSET		13
#define USBA_ISO_ERR_NBTRANS_SIZE		1
#define USBA_NAK_IN_OFFSET			14
#define USBA_NAK_IN_SIZE			1
#define USBA_ISO_ERR_FLUSH_OFFSET		14
#define USBA_ISO_ERR_FLUSH_SIZE			1
#define USBA_NAK_OUT_OFFSET			15
#define USBA_NAK_OUT_SIZE			1
#define USBA_CURRENT_BANK_OFFSET		16
#define USBA_CURRENT_BANK_SIZE			2
#define USBA_BUSY_BANKS_OFFSET			18
#define USBA_BUSY_BANKS_SIZE			2
#define USBA_BYTE_COUNT_OFFSET			20
#define USBA_BYTE_COUNT_SIZE			11
#define USBA_SHORT_PACKET_OFFSET		31
#define USBA_SHORT_PACKET_SIZE			1

/* Bitfields in DMA_CONTROL */
#define USBA_DMA_CH_EN_OFFSET			0
#define USBA_DMA_CH_EN_SIZE			1
#define USBA_DMA_LINK_OFFSET			1
#define USBA_DMA_LINK_SIZE			1
#define USBA_DMA_END_TR_EN_OFFSET		2
#define USBA_DMA_END_TR_EN_SIZE			1
#define USBA_DMA_END_BUF_EN_OFFSET		3
#define USBA_DMA_END_BUF_EN_SIZE		1
#define USBA_DMA_END_TR_IE_OFFSET		4
#define USBA_DMA_END_TR_IE_SIZE			1
#define USBA_DMA_END_BUF_IE_OFFSET		5
#define USBA_DMA_END_BUF_IE_SIZE		1
#define USBA_DMA_DESC_LOAD_IE_OFFSET		6
#define USBA_DMA_DESC_LOAD_IE_SIZE		1
#define USBA_DMA_BURST_LOCK_OFFSET		7
#define USBA_DMA_BURST_LOCK_SIZE		1
#define USBA_DMA_BUF_LEN_OFFSET			16
#define USBA_DMA_BUF_LEN_SIZE			16

/* Bitfields in DMA_STATUS */
#define USBA_DMA_CH_ACTIVE_OFFSET		1
#define USBA_DMA_CH_ACTIVE_SIZE			1
#define USBA_DMA_END_TR_ST_OFFSET		4
#define USBA_DMA_END_TR_ST_SIZE			1
#define USBA_DMA_END_BUF_ST_OFFSET		5
#define USBA_DMA_END_BUF_ST_SIZE		1
#define USBA_DMA_DESC_LOAD_ST_OFFSET		6
#define USBA_DMA_DESC_LOAD_ST_SIZE		1

/* Constants for SPEED_CFG */
#define USBA_SPEED_CFG_NORMAL			0
#define USBA_SPEED_CFG_FORCE_HIGH		2
#define USBA_SPEED_CFG_FORCE_FULL		3

/* Constants for EPT_SIZE */
#define USBA_EPT_SIZE_8				0
#define USBA_EPT_SIZE_16			1
#define USBA_EPT_SIZE_32			2
#define USBA_EPT_SIZE_64			3
#define USBA_EPT_SIZE_128			4
#define USBA_EPT_SIZE_256			5
#define USBA_EPT_SIZE_512			6
#define USBA_EPT_SIZE_1024			7

/* Constants for EPT_TYPE */
#define USBA_EPT_TYPE_CONTROL			0
#define USBA_EPT_TYPE_ISO			1
#define USBA_EPT_TYPE_BULK			2
#define USBA_EPT_TYPE_INT			3

/* Constants for BK_NUMBER */
#define USBA_BK_NUMBER_ZERO			0
#define USBA_BK_NUMBER_ONE			1
#define USBA_BK_NUMBER_DOUBLE			2
#define USBA_BK_NUMBER_TRIPLE			3

/* Bit manipulation macros */
#define USBA_BIT(name)						\
	(1 << USBA_##name##_OFFSET)
#define USBA_BF(name, value)					\
	(((value) & ((1 << USBA_##name##_SIZE) - 1))		\
	 << USBA_##name##_OFFSET)
#define USBA_BFEXT(name, value)					\
	(((value) >> USBA_##name##_OFFSET)			\
	 & ((1 << USBA_##name##_SIZE) - 1))
#define USBA_BFINS(name, value, old)				\
	(((old) & ~(((1 << USBA_##name##_SIZE) - 1)		\
		    << USBA_##name##_OFFSET))			\
	 | USBA_BF(name, value))

/* Register access macros */
#define usba_readl(udc, reg)					\
	__raw_readl((udc)->regs + USBA_##reg)
#define usba_writel(udc, reg, value)				\
	__raw_writel((value), (udc)->regs + USBA_##reg)
#define usba_ep_readl(ep, reg)					\
	__raw_readl((ep)->ep_regs + USBA_EPT_##reg)
#define usba_ep_writel(ep, reg, value)				\
	__raw_writel((value), (ep)->ep_regs + USBA_EPT_##reg)
#define usba_dma_readl(ep, reg)					\
	__raw_readl((ep)->dma_regs + USBA_DMA_##reg)
#define usba_dma_writel(ep, reg, value)				\
	__raw_writel((value), (ep)->dma_regs + USBA_DMA_##reg)

/* Calculate base address for a given endpoint or DMA controller */
#define USBA_EPT_BASE(x)	(0x100 + (x) * 0x20)
#define USBA_DMA_BASE(x)	(0x300 + (x) * 0x10)
#define USBA_FIFO_BASE(x)	((x) << 16)

/* Synth parameters */
#define USBA_NR_ENDPOINTS	7

#define EP0_FIFO_SIZE		64
#define EP0_EPT_SIZE		USBA_EPT_SIZE_64
#define EP0_NR_BANKS		1
#define BULK_FIFO_SIZE		512
#define BULK_EPT_SIZE		USBA_EPT_SIZE_512
#define BULK_NR_BANKS		2
#define ISO_FIFO_SIZE		1024
#define ISO_EPT_SIZE		USBA_EPT_SIZE_1024
#define ISO_NR_BANKS		3
#define INT_FIFO_SIZE		64
#define INT_EPT_SIZE		USBA_EPT_SIZE_64
#define INT_NR_BANKS		3

enum usba_ctrl_state {
	WAIT_FOR_SETUP,
	DATA_STAGE_IN,
	DATA_STAGE_OUT,
	STATUS_STAGE_IN,
	STATUS_STAGE_OUT,
	STATUS_STAGE_ADDR,
	STATUS_STAGE_TEST,
};
/*
  EP_STATE_IDLE,
  EP_STATE_SETUP,
  EP_STATE_IN_DATA,
  EP_STATE_OUT_DATA,
  EP_STATE_SET_ADDR_STATUS,
  EP_STATE_RX_STATUS,
  EP_STATE_TX_STATUS,
  EP_STATE_HALT,
*/

struct usba_dma_desc {
	dma_addr_t next;
	dma_addr_t addr;
	u32 ctrl;
};

struct usba_ep {
	int					state;
	void __iomem				*ep_regs;
	void __iomem				*dma_regs;
	void __iomem				*fifo;
	struct usb_ep				ep;
	struct usba_udc				*udc;

	struct list_head			queue;
	const struct usb_endpoint_descriptor	*desc;

	u16					fifo_size;
	u8					nr_banks;
	u8					index;
	unsigned int				can_dma:1;
	unsigned int				can_isoc:1;
	unsigned int				is_isoc:1;
	unsigned int				is_in:1;

#ifdef CONFIG_DEBUG_FS
	u32					last_dma_status;
	struct dentry				*debugfs_dir;
	struct dentry				*debugfs_queue;
	struct dentry				*debugfs_dma_status;
	struct dentry				*debugfs_state;
#endif
};

struct usba_request {
	struct usb_request			req;
	struct list_head			queue;

	u32					ctrl;

	unsigned int				submitted:1;
	unsigned int				last_transaction:1;
	unsigned int				using_dma:1;
	unsigned int				mapped:1;
};

struct usba_udc {
	/* Protect hw registers from concurrent modifications */
	spinlock_t lock;

	void __iomem *regs;
	void __iomem *fifo;

	struct usb_gadget gadget;
	struct usb_gadget_driver *driver;
	struct platform_device *pdev;
	int irq;
	int vbus_pin;
	struct clk *pclk;
	struct clk *hclk;

	int test_mode;
	int vbus_prev;

#ifdef CONFIG_DEBUG_FS
	struct dentry *debugfs_root;
	struct dentry *debugfs_regs;
#endif
};

#define to_usba_ep(x) container_of((x), struct usba_ep, ep)
#define to_usba_req(x) container_of((x), struct usba_request, req)
#define to_usba_udc(x) container_of((x), struct usba_udc, gadget)

#define ep_index(ep)		((ep)->index)
#define ep_can_dma(ep)		((ep)->can_dma)
#define ep_is_in(ep)		((ep)->is_in)
#define ep_is_isochronous(ep)	((ep)->is_isoc)
#define ep_is_control(ep)	(ep_index(ep) == 0)
#define ep_name(ep)		((ep)->ep.name)
#define ep_is_idle(ep)		((ep)->state == EP_STATE_IDLE)

#endif /* __LINUX_USB_GADGET_USBA_UDC_H */
